export { UnpatchDirective, UnpatchModule as UnpatchEventsModule } from '@rx-angular/template/unpatch';
import { Directive, ElementRef, Inject, Optional, NgModule } from '@angular/core';
import { RxStrategyProvider } from '@rx-angular/cdk/render-strategies';
import { LetDirective } from '@rx-angular/template/let';
export { LetDirective, LetModule } from '@rx-angular/template/let';
import { Subject, Observable } from 'rxjs';
import { mergeAll, map, filter, withLatestFrom } from 'rxjs/operators';
export { PushModule, PushPipe } from '@rx-angular/template/push';

function intersectionObserver(options) {
    const subject = new Subject();
    const observer = observerSupported()
        ? new IntersectionObserver((entries) => {
            entries.forEach((entry) => subject.next(entry));
        }, options)
        : null;
    const entries$ = new Observable((subscriber) => {
        subject.subscribe(subscriber);
        return () => {
            if (observer) {
                observer.disconnect();
            }
        };
    });
    return {
        entries$,
        observe: observer.observe,
        unobserve: observer.unobserve,
    };
}
const observerSupported = () => typeof window !== 'undefined'
    ? !!window.IntersectionObserver
    : false;
const ɵ0 = observerSupported;
class ViewportPrioDirective {
    constructor(el, strategyProvider, letDirective) {
        this.el = el;
        this.strategyProvider = strategyProvider;
        this.letDirective = letDirective;
        // Note that we're picking only the `intersectionRatio` property
        // since this is the only property that we're intersted in.
        this.entriesSubject = new Subject();
        this.entries$ = this.entriesSubject.pipe(mergeAll());
        this._viewportPrio = 'noop';
        /* @Input('viewport-prio')
        set viewportPrio(prio) {
          if (prio) {
            this._viewportPrio = prio || 'noop';
          }
        }*/
        this.observer = observerSupported()
            ? new IntersectionObserver((entries) => {
                this.entriesSubject.next(entries);
            }, {
                threshold: 0,
            })
            : null;
        this.visibilityEvents$ = this.entries$.pipe(map((entry) => {
            if (entry.intersectionRatio > 0) {
                return 'visible';
            }
            else {
                return 'invisible';
            }
        }));
    }
    ngOnInit() {
        const letStrategyName$ = this.letDirective['strategyHandler'].values$.pipe(filter((name) => name !== this._viewportPrio));
        this.visibilityEvents$
            .pipe(withLatestFrom(letStrategyName$), map(([visibility, strategyName]) => visibility === 'visible' ? strategyName : this._viewportPrio))
            .subscribe((strategyName) => {
            if (this.letDirective !== null) {
                this.letDirective.strategy = strategyName;
            }
            // render actual state on viewport enter
            // @TODO this doesnt catch unsubscribe (cant be cancelled)
            // @TODO: we need to fetch the current template of the letDirective here
            // this.strategyProvider.scheduleCD()
        });
        // If the browser doesn't support the `IntersectionObserver` or we're inside
        // the Node.js environment, then this will throw an exception that property
        // `observe` doesn't exist on `null`.
        if (this.observer !== null) {
            this.observer.observe(this.el.nativeElement);
        }
        else {
            // If we're inside the Node.js environment then this should be
            // rendered (e.g. for SEO purposes), and when running this code in browser
            // it will decide itself to render it or not.
            this.entriesSubject.next([{ intersectionRatio: 1 }]);
        }
    }
    ngOnDestroy() {
        if (this.observer) {
            this.observer.disconnect();
        }
    }
}
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
ViewportPrioDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[viewport-prio]',
            },] }
];
/**
 * @type {function(): !Array<(null|{
 *   type: ?,
 *   decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>),
 * })>}
 * @nocollapse
 */
ViewportPrioDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: RxStrategyProvider },
    { type: LetDirective, decorators: [{ type: Inject, args: [LetDirective,] }, { type: Optional }] }
];

const DECLARATIONS = [ViewportPrioDirective];
class ViewportPrioModule {
}
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
ViewportPrioModule.decorators = [
    { type: NgModule, args: [{
                declarations: DECLARATIONS,
                exports: DECLARATIONS
            },] }
];

// EXPERIMENTAL

/**
 * Generated bundle index. Do not edit.
 */

export { ViewportPrioDirective, ViewportPrioModule };
//# sourceMappingURL=template.js.map
