"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const ts = require("typescript");
const changes_1 = require("../../common/utils/changes");
const format_files_1 = require("../../common/utils/format-files");
const insert_1 = require("../../common/utils/insert");
const replace_node_value_1 = require("../../common/utils/replace-node-value");
const visitors_1 = require("../../common/utils/visitors");
const renames = {
    LetModule: '@rx-angular/template/let',
    LetDirective: '@rx-angular/template/let',
    PushModule: '@rx-angular/template/push',
    PushPipe: '@rx-angular/template/push',
    UnpatchDirective: '@rx-angular/template/unpatch',
    UnpatchEventsModule: ['UnpatchModule', '@rx-angular/template/unpatch'],
};
function default_1() {
    return schematics_1.chain([
        (tree) => {
            visitors_1.visitTSSourceFiles(tree, (sourceFile) => {
                /* Collect RxAngular imports. */
                const imports = sourceFile.statements
                    .filter(ts.isImportDeclaration)
                    .filter(({ moduleSpecifier }) => moduleSpecifier.getText(sourceFile) ===
                    `'@rx-angular/template'` ||
                    moduleSpecifier.getText(sourceFile) === `"@rx-angular/template"`);
                if (imports.length === 0) {
                    return;
                }
                /* Remove old imports. */
                const removeChanges = findImportSpecifiers(sourceFile, imports).map(({ importDeclaration }) => {
                    return changes_1.createRemoveChange(sourceFile, importDeclaration, importDeclaration.getStart(), importDeclaration.getFullText());
                });
                /* Insert new imports. */
                const insertChanges = findImportSpecifiers(sourceFile, imports).map(({ importSpecifier }) => {
                    const rename = renames[importSpecifier];
                    return insert_1.insertImport(sourceFile, sourceFile.fileName, typeof rename === 'string' ? importSpecifier : rename[0], typeof rename === 'string' ? rename : rename[1]);
                });
                insert_1.insert(tree, sourceFile.fileName, [...insertChanges, ...removeChanges]);
            });
        },
        (tree) => {
            visitors_1.visitTSSourceFiles(tree, (sourceFile) => {
                /* Replace UnpatchEventsModule declaration to UnpatchModule. */
                (function replaceUnpatchEventsModule(node) {
                    if (ts.isIdentifier(node) &&
                        node.getText(sourceFile) === 'UnpatchEventsModule') {
                        replace_node_value_1.replaceNodeValue(tree, sourceFile.fileName, node, renames.UnpatchEventsModule[0] + ',');
                    }
                    ts.forEachChild(node, replaceUnpatchEventsModule);
                })(sourceFile);
            });
        },
        format_files_1.formatFiles()
    ]);
}
exports.default = default_1;
function findImportSpecifiers(sourceFile, imports) {
    return imports.flatMap((importDeclaration) => {
        const importSpecifiers = ast_utils_1.findNodes(importDeclaration, ts.SyntaxKind.ImportSpecifier);
        return importSpecifiers.map((importSpecifier) => ({
            importDeclaration,
            importSpecifier: importSpecifier.getText(sourceFile),
        }));
    });
}
